@file:JvmName("AuthApiClientKt")

package com.kakao.sdk.partner.auth

import com.kakao.sdk.auth.AuthApiClient
import com.kakao.sdk.auth.TokenManagerProvider
import com.kakao.sdk.auth.model.AccessTokenResponse
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.network.kauth
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.ApplicationInfo
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.ContextInfo
import com.kakao.sdk.common.network.ApiFactory
import retrofit2.Call
import retrofit2.Callback
import retrofit2.HttpException
import retrofit2.Response

fun AuthApiClient.issueAccessTokenForGroup(
    groupRefreshToken: String,
    callback: (token: OAuthToken?, error: Throwable?) -> Unit
) {
    PartnerAuthApiClient.instance
        .issueAccessTokenForGroup(groupRefreshToken, callback)
}

/**
 * @suppress
 */
class PartnerAuthApiClient(
    private val authApi: PartnerAuthApi = ApiFactory.kauth.create(PartnerAuthApi::class.java),
    private val tokenManagerProvider: TokenManagerProvider = TokenManagerProvider.instance,
    private val applicationInfo: ApplicationInfo = KakaoSdk.applicationContextInfo,
    private val contextInfo: ContextInfo = KakaoSdk.applicationContextInfo
) {
    fun issueAccessTokenForGroup(
        groupRefreshToken: String,
        callback: (token: OAuthToken?, error: Throwable?) -> Unit
    ) {
        val clientId = applicationInfo.appKey
        val androidKeyHash = contextInfo.signingKeyHash
        authApi.issueAccessTokenForGroup(
            groupRefreshToken,
            clientId = clientId,
            androidKeyHash = androidKeyHash
        ).enqueue(object: Callback<AccessTokenResponse> {
            override fun onFailure(call: Call<AccessTokenResponse>, t: Throwable) {
                callback(null, t)
            }

            override fun onResponse(
                call: Call<AccessTokenResponse>,
                response: Response<AccessTokenResponse>
            ) {
                if (response.isSuccessful) {
                    response.body()?.let {
                        val token = OAuthToken.fromResponse(it)
                        tokenManagerProvider.manager.setToken(token)
                        callback(token, null)
                        return
                    }
                    callback(null, ClientError(ClientErrorCause.Unknown, "No body"))
                } else {
                    callback(null, AuthApiClient.translateError(HttpException(response)))
                }
            }
        })
    }

    companion object {
        /**
         * 간편한 API 호출을 위해 기본 제공되는 singleton 객체
         */
        @JvmStatic
        val instance by lazy { PartnerAuthApiClient() }
    }
}